/*
  Copyright (c) 2015, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "generator.hpp"

#include <cstdio>


namespace solar {

class output_t {
private:
	std::string prefix_str;
	std::string token_prefix_str;
	unsigned stack_size;

	const generator_t *generator;

	std::FILE *header_file;
	std::FILE *source_file;

	std::map<std::string, unsigned> tokens;
	std::map<std::string, unsigned> nonterms;

	const char * prefix() const {
		return prefix_str.c_str();
	}

	const char * token_prefix() const {
		return token_prefix_str.c_str();
	}

	void emit_tokens();
	void emit_token_value();
	void emit_header();

	void emit_state_shift(unsigned i);
	void emit_state_reduce(const item_t &item);
	void emit_state(unsigned i);
	void emit_states();
	void emit_source();

public:
	output_t(const generator_t *generator0, const char *header, const char *source);

	void write();
};

}
